SuperStrict

Framework BRL.Max2D
Import BRL.GLMax2D
Import BRL.Retro
Import BRL.PNGLoader

Type TWorld
	Const MAPWIDTH:Int = 32
	Const MAPHEIGHT:Int = 20
	Const TILEWIDTH:Int = 32
	Const TILEHEIGHT:Int = 32
	Global Gras:TImage
	Global Water:TImage
	Global TileMap:Int[MAPWIDTH, MAPHEIGHT]
	Global CursorPos:TPoint = New TPoint
	Field WaterMap:Int[MAPWIDTH * 2, MAPHEIGHT * 2]
	Field Scroll:TPoint = New TPoint
	
	Function Load(map:String)
		Gras = LoadImage("graphics/gras.png", 0)
		Water = LoadAnimImage("graphics/water.png", TILEWIDTH * 0.5, TILEHEIGHT * 0.5, 0, 32, MASKEDIMAGE)
		
		Local stream:TStream = ReadStream(map)
		If stream <> Null Then
			For Local i:Int = 0 Until MAPWIDTH
				For Local j:Int = 0 Until MAPHEIGHT
					TileMap[i, j] = stream.ReadByte()
				Next
			Next
			stream.Close()
		EndIf
	End Function
	
	Function Save(map:String)
		Local stream:TStream = WriteStream(map)
		For Local i:Int = 0 Until MAPWIDTH
			For Local j:Int = 0 Until MAPHEIGHT
				stream.WriteByte(TileMap[i, j])
			Next
		Next
		stream.Close()
	End Function
	
	Function GetTile:Int(x:Int, y:Int)
		If x < 0 Or x >= MAPWIDTH Then Return 1
		If y < 0 Or y >= MAPHEIGHT Then Return 1
		Return TileMap[x, y]
	End Function
	
	Function SetTile(tile:Int, x:Int, y:Int)
		If x < 0 Or x >= MAPWIDTH Then Return
		If y < 0 Or y >= MAPHEIGHT Then Return
		TileMap[x, y] = tile
	End Function
	
	Method UpdateWaterMap()
		For Local i:Int = 0 Until MAPWIDTH
			For Local j:Int = 0 Until MAPHEIGHT
				Self.WaterMap[i * 2, j * 2] = (GetTile(i - 1, j) * 4 + GetTile(i - 1, j - 1) * 2 + GetTile(i, j - 1)) * 2
				Self.WaterMap[i * 2 + 1, j * 2] = (GetTile(i + 1 , j) * 4 + GetTile(i + 1 , j - 1) * 2 + GetTile(i , j - 1)) * 2 + 1
				Self.WaterMap[i * 2, j * 2 + 1] = (GetTile(i - 1 , j) * 4 + GetTile(i - 1 , j + 1) * 2 + GetTile(i , j + 1)) * 2 + 16
				Self.WaterMap[i * 2 + 1, j * 2 + 1] = (GetTile(i + 1 , j) * 4 + GetTile(i + 1 , j + 1) * 2 + GetTile(i , j + 1)) * 2 + 17
			Next
		Next
	End Method
	
	Method Update()
		Local fx:Float = Float(MouseX()) / GraphicsWidth(), fy:Float = Float(MouseY()) / GraphicsHeight()
		Self.Scroll = TPoint.Create(GraphicsWidth() * 0.5 - fx * MAPWIDTH * TILEWIDTH, GraphicsHeight() * 0.5 - fy * MAPHEIGHT * TILEHEIGHT)
		Self.Scroll.X = Min(Self.Scroll.X, 0)
		Self.Scroll.Y = Min(Self.Scroll.Y, 0)
		Self.Scroll.X = Max(Self.Scroll.X, - MAPWIDTH * TILEWIDTH + GraphicsWidth())
		Self.Scroll.Y = Max(Self.Scroll.Y, - MAPHEIGHT * TILEHEIGHT + GraphicsHeight())
		CursorPos = TPoint.Create((MouseX() - Self.Scroll.X) / 32, (MouseY() - Self.Scroll.Y) / 32)
		If MouseDown(MOUSE_LEFT) Then SetTile(1, Self.CursorPos.X, Self.CursorPos.Y) Self.UpdateWaterMap()
		If MouseDown(MOUSE_RIGHT) Then SetTile(0, Self.CursorPos.X, Self.CursorPos.Y) Self.UpdateWaterMap()
	End Method
	
	Method RenderMiniMap(x:Int, y:Int)
		SetAlpha 0.6
		For Local i:Int = 0 Until MAPWIDTH
			For Local j:Int = 0 Until MAPHEIGHT
				If TileMap[i, j] = 1 Then SetColor 100, 160, 240 DrawRect x + i * 2, y + j * 2, 2, 2
			Next
		Next
		SetColor 255, 255, 255
		SetAlpha 1
	End Method
	
	Method Render()
		TileImage Gras, Self.Scroll.X, Self.Scroll.Y
		
		SetAlpha 0.4
		For Local i:Int = 0 Until MAPWIDTH
			For Local j:Int = 0 Until MAPHEIGHT
				If TileMap[i, j] > 0 Then
					For Local si:Int = 0 Until 2
						For Local sj:Int = 0 Until 2
							DrawImage Water, Self.Scroll.X + (i + si * 0.5) * TILEWIDTH, Self.Scroll.Y + (j + sj * 0.5) * TILEHEIGHT, Self.WaterMap[i * 2 + si, j * 2 + sj]
						Next
					Next
				EndIf
			Next
		Next
		SetAlpha 1
		
		Self.RenderMiniMap(GraphicsWidth() - MAPWIDTH * 2, 0)
	End Method
End Type

Type TPoint
	Field X:Int, Y:Int
	
	Function Create:TPoint(x:Int, y:Int)
		Local created:TPoint = New TPoint
		created.X = x
		created.Y = y
		Return created
	End Function
End Type

AppTitle:String = "Horstcraft I - MapEditor"
SetGraphicsDriver GLMax2DDriver()
Graphics 640, 400, 0, 60
SetBlend ALPHABLEND
SetMaskColor 127, 127, 127

Local map:String = "default.map"
If AppArgs.Dimensions()[0] > 1 Then map = AppArgs[1]
TWorld.Load("maps/" + map)
Local world:TWorld = New TWorld
world.UpdateWaterMap()

Repeat
	Cls
	world.Update()
	world.Render()
	If KeyHit(KEY_S) Then TWorld.Save("maps/" + map)
	Flip
Until AppTerminate()
