SuperStrict

Framework BRL.Max2D
Import BRL.GLMax2D
Import BRL.Retro
Import BRL.PNGLoader
Import BRL.FreeTypeFont

Type TWorld
	Const MAPWIDTH:Int = 32, MAPHEIGHT:Int = 20
	Const TILEWIDTH:Int = 32, TILEHEIGHT:Int = 32
	Global Messages:String[] = ["Blue's turn", "Red's turn", "Red wins!", "Blue wins!"]
	Global Gras:TImage, Water:TImage
	Global TileMap:Int[MAPWIDTH, MAPHEIGHT], UnitMap:TUnit[MAPWIDTH, MAPHEIGHT]
	Global Particles:TList = CreateList()
	Field WaterMap:Int[MAPWIDTH * 2, MAPHEIGHT * 2]
	Global Cursor:TPoint = New TPoint
	Field PlayersTurn:Int, SelUnit:TUnit
	Field Scroll:TPoint = New TPoint
	
	Function Load(map:String)
		Local i:Int, j:Int, x:Int, y:Int
		Gras = LoadImage("graphics/gras.png", 0)
		Water = LoadAnimImage("graphics/water.png", TILEWIDTH * 0.5, TILEHEIGHT * 0.5, 0, 32, MASKEDIMAGE)
		TUnit.Load()
		
		Local stream:TStream = ReadStream(map)
		If stream <> Null Then
			For Local i:Int = 0 Until MAPWIDTH
				For Local j:Int = 0 Until MAPHEIGHT
					TileMap[i, j] = stream.ReadByte()
				Next
			Next
			stream.Close()
		EndIf
		
		For i = 0 Until 5
			For j = 0 Until 2
				Repeat
					x = Rnd(MAPWIDTH) * 0.5 + 16 * j
					y = Rnd(MAPHEIGHT)
				Until FieldIsFree(x, y)
				UnitMap[x, y] = TUnit.Create(j, Rnd(3), x, y)
			Next
		Next
		CheckPaths()
	End Function
	
	Function CheckPaths()
		For Local i:Int = 0 Until MAPWIDTH
			For Local j:Int = 0 Until MAPHEIGHT
				If UnitMap[i, j] <> Null Then UnitMap[i, j].CheckPaths()
			Next
		Next
	End Function
	
	Function FieldIsFree:Byte(x:Int, y:Int)
		If x < 0 Or x >= MAPWIDTH Then Return False
		If y < 0 Or y >= MAPHEIGHT Then Return False
		If TileMap[x, y] > 0 Then Return False
		If UnitMap[x, y] <> Null Then Return False
		Return True
	End Function
	
	Function GetTile:Int(x:Int, y:Int)
		If x < 0 Or x >= MAPWIDTH Then Return 1
		If y < 0 Or y >= MAPHEIGHT Then Return 1
		Return TileMap[x, y]
	End Function
	
	Method UpdateWaterMap()
		For Local i:Int = 0 Until MAPWIDTH
			For Local j:Int = 0 Until MAPHEIGHT
				Self.WaterMap[i * 2, j * 2] = (GetTile(i - 1, j) * 4 + GetTile(i - 1, j - 1) * 2 + GetTile(i, j - 1)) * 2
				Self.WaterMap[i * 2 + 1, j * 2] = (GetTile(i + 1 , j) * 4 + GetTile(i + 1 , j - 1) * 2 + GetTile(i , j - 1)) * 2 + 1
				Self.WaterMap[i * 2, j * 2 + 1] = (GetTile(i - 1 , j) * 4 + GetTile(i - 1 , j + 1) * 2 + GetTile(i , j + 1)) * 2 + 16
				Self.WaterMap[i * 2 + 1, j * 2 + 1] = (GetTile(i + 1 , j) * 4 + GetTile(i + 1 , j + 1) * 2 + GetTile(i , j + 1)) * 2 + 17
			Next
		Next
	End Method
	
	Method NextPlayer()
		Self.PlayersTurn = 1 - Self.PlayersTurn
		Self.SelUnit = Null
		For Local i:Int = 0 Until MAPWIDTH
			For Local j:Int = 0 Until MAPHEIGHT
				Local unit:TUnit = UnitMap[i, j]
				If unit <> Null Then unit.Moved = False unit.Attacked = False
			Next
		Next
	End Method
	
	Method Update()
		Local unit:TUnit
		Local fx:Float = Float(MouseX()) / GraphicsWidth(), fy:Float = Float(MouseY()) / GraphicsHeight()
		Self.Scroll = TPoint.Create(GraphicsWidth() * 0.5 - fx * MAPWIDTH * TILEWIDTH, GraphicsHeight() * 0.5 - fy * MAPHEIGHT * TILEHEIGHT)
		Self.Scroll.X = Min(Self.Scroll.X, 0)
		Self.Scroll.Y = Min(Self.Scroll.Y, 0)
		Self.Scroll.X = Max(Self.Scroll.X, - MAPWIDTH * TILEWIDTH + GraphicsWidth())
		Self.Scroll.Y = Max(Self.Scroll.Y, - MAPHEIGHT * TILEHEIGHT + GraphicsHeight())
		Cursor = TPoint.Create((MouseX() - Self.Scroll.X) / 32, (MouseY() - Self.Scroll.Y) / 32)
		
		If Self.PlayersTurn <= 1 Then
			If KeyHit(KEY_SPACE) Then Self.NextPlayer()
			
			If MouseHit(MOUSE_RIGHT) And Self.SelUnit <> Null Then
				If Self.SelUnit.Team = Self.PlayersTurn Then
					unit = UnitMap[Cursor.X, Cursor.Y]
					If unit <> Null Then
						If Self.SelUnit.Attacked = False Then
							If Abs(Self.SelUnit.Pos.X - unit.Pos.X) + Abs(Self.SelUnit.Pos.Y - unit.Pos.Y) = 1 Then
								Self.SelUnit.Attack(unit)
								If unit.Health = 0 Then UnitMap[Cursor.X, Cursor.Y] = Null
								CheckPaths()
							EndIf
						EndIf
					EndIf
				EndIf
			EndIf
			
			If MouseHit(MOUSE_LEFT) Then
				If Self.SelUnit <> Null Then
					If Self.SelUnit.Paths[Cursor.X, Cursor.Y] <> Null And Self.SelUnit.Team = Self.PlayersTurn And Self.SelUnit.Moved = False Then
						Self.SelUnit.Move(Cursor.X, Cursor.Y)
					Else
						Self.SelUnit = UnitMap[Cursor.X, Cursor.Y]
					EndIf
				Else
					Self.SelUnit = UnitMap[Cursor.X, Cursor.Y]
				EndIf
			EndIf
			
			Local moveable:Int, alive:Int
			For Local i:Int = 0 Until MAPWIDTH
				For Local j:Int = 0 Until MAPHEIGHT
					unit = UnitMap[i, j]
					If unit <> Null Then
						unit.Update()
						If unit.Team = Self.PlayersTurn Then alive:+ 1 If unit.Moved = False Then moveable:+ 1
					EndIf
				Next
			Next
			If alive = 0 Then
				Self.PlayersTurn:+ 2
			Else
				If moveable = 0 Then Self.NextPlayer()
			EndIf
		EndIf
		
		For Local particle:TParticle = EachIn Particles
			particle.Update()
		Next
	End Method
	
	Method RenderMiniMap(x:Int, y:Int)
		SetAlpha 0.6
		For Local i:Int = 0 Until MAPWIDTH
			For Local j:Int = 0 Until MAPHEIGHT
				SetColor 255, 255, 255
				If TileMap[i, j] = 1 Then SetColor 0, 0, 0
				If UnitMap[i, j] <> Null Then
					SetColor 80, 100, 200
					If UnitMap[i, j].Team = 1 Then SetColor 200, 100, 80
				EndIf
				DrawRect x + i * 2, y + j * 2, 2, 2
			Next
		Next
		
		For Local particle:TParticle = EachIn Particles
			particle.Render(Self.Scroll.X, Self.Scroll.Y)
		Next
		SetColor 255, 255, 255
		SetAlpha 1
	End Method
	
	Method Render()
		Local selpath:TNode
		TileImage Gras, Self.Scroll.X, Self.Scroll.Y
		
		If Self.SelUnit <> Null Then
			If Self.SelUnit.Moved = False And Self.SelUnit.Team = Self.PlayersTurn Then selpath = Self.SelUnit.Paths[Cursor.X, Cursor.Y]
		EndIf
		For Local i:Int = 0 Until MAPWIDTH
			For Local j:Int = 0 Until MAPHEIGHT
				If TileMap[i, j] > 0 Then
					For Local si:Int = 0 Until 2
						For Local sj:Int = 0 Until 2
							SetAlpha 0.4
							DrawImage Water, Self.Scroll.X + (i + si * 0.5) * TILEWIDTH, Self.Scroll.Y + (j + sj * 0.5) * TILEHEIGHT, Self.WaterMap[i * 2 + si, j * 2 + sj]
						Next
					Next
				EndIf
				SetAlpha 1
				If UnitMap[i, j] <> Null Then UnitMap[i, j].Render(Self.Scroll.X, Self.Scroll.Y)
				If Self.SelUnit <> Null Then
					If selpath = Null And Self.SelUnit.Moved = False Then
						SetColor 80, 100, 200
						If Self.SelUnit.Team = 1 Then SetColor 200, 100, 80
						If Self.SelUnit.Paths[i, j] <> Null Then	Self.SelUnit.Paths[i, j].Render(False, Self.Scroll.X, Self.Scroll.Y)
						SetColor 255, 255, 255
					EndIf
				EndIf
			Next
		Next
		If selpath <> Null Then selpath.Render(True, Self.Scroll.X, Self.Scroll.Y)
		
		Local info:String = Messages[Self.PlayersTurn]
		DrawTextOutline info, GraphicsWidth() * 0.5 - TextWidth(info) * 0.5, 16
		SetColor 255, 255, 255
		DrawText info, GraphicsWidth() * 0.5 - TextWidth(info) * 0.5, 16
		
		Self.RenderMiniMap(GraphicsWidth() - MAPWIDTH * 2, 0)
	End Method
End Type

Type TUnit
	Global Sprite:TImage
	Field Team:Int, Char:Int, Health:Int = 100
	Field Pos:TPoint = New TPoint, SPos:TPoint, Range:Int = 4
	Field Paths:TNode[TWorld.MAPWIDTH, TWorld.MAPHEIGHT], Path:TNode, Node:TNode
	Field Moved:Byte, Attacked:Byte
	
	Function Load()
		Sprite = LoadAnimImage("graphics/units.png", 32, 32, 0, 3)
		TNode.Load()
	End Function
	
	Function Create:TUnit(team:Int, char:Int, x:Int, y:Int)
		Local created:TUnit = New TUnit
		created.Team = team
		created.Char = char
		created.Pos = TPoint.Create(x, y)
		created.SPos = TPoint.Create(x * TWorld.TILEWIDTH, y * TWorld.TILEHEIGHT)
		Return created
	End Function
	
	Method CheckPaths()
		Local nodes:TList = CreateList(), nextnodes:TList = CreateList()
		nodes.AddLast(TNode.Create(Null, Self.Pos.X, Self.Pos.Y))
		
		Self.Paths = New TNode[TWorld.MAPWIDTH, TWorld.MAPHEIGHT]
		For Local i:Int = 0 Until Self.Range
			For Local node:TNode = EachIn nodes
				For Local alpha:Int = 0 Until 360 Step 90
					Local x:Int = node.Pos.X + Sin(alpha)
					Local y:Int = node.Pos.Y - Cos(alpha)
					If TWorld.FieldIsFree(x, y) Then
						If Self.Paths[x, y] = Null Then
							Local nextnode:TNode = TNode.Create(node, x, y)
							Self.Paths[x, y] = nextnode
							nextnodes.AddLast(nextnode)
						EndIf
					EndIf
				Next
			Next
			nodes = nextnodes.Copy()
			nextnodes.Clear()
		Next
	End Method
	
	Method Move(x:Int, y:Int)
		Self.Path = Self.Paths[x, y]
		TWorld.UnitMap[Self.Pos.X, Self.Pos.Y] = Null
		Self.Pos = TPoint.Create(x, y)
		TWorld.UnitMap[Self.Pos.X, Self.Pos.Y] = Self
		TWorld.CheckPaths()
		Self.Moved = True
	End Method
	
	Method Attack(target:TUnit)
		Local damage:Float = 30 + Rnd(10)
		If Self.Char - target.Char = - 1 Or (Self.Char = 2 And target.Char = 0) Then
			damage:* 1.5
		ElseIf Self.Char - target.Char = 1 Or (Self.Char = 0 And target.Char = 2) Then
			damage:* 0.5
		EndIf
		target.Health = Max(target.Health - damage, 0)
		Local r:Int = 0, g:Int = 0, b:Int = 0
		For Local i:Int = 0 Until 50
			If target.Health > 0 Then r = 200 g = 100 b = 80
			TParticle.Create(target.SPos.X + Rnd(TWorld.TILEWIDTH), target.SPos.Y + Rnd(TWorld.TILEHEIGHT), Rand(3) - 2, Rand(3) - 2, r, g, b)
		Next
		Self.Attacked = True
		Self.Moved = True
	End Method
	
	Method Update()
		If Self.Path <> Null And Self.Node = Null Then
			If Self.Path.Parent = Null Then
				Self.Node = Self.Path
				Self.Path = Null
			Else
				Local search:TNode = Self.Path
				Local before:TNode = Self.Path
				While search.Parent <> Null
					before = search
					search = search.Parent
				Wend
				Self.Node = before.Parent
				before.Parent = Null
			EndIf
		EndIf
		
		If Self.Node <> Null Then
			If Self.Node.Pos.X * TWorld.TILEWIDTH < Self.SPos.X Then Self.SPos.X:- 4
			If Self.Node.Pos.X * TWorld.TILEWIDTH > Self.SPos.X Then Self.SPos.X:+ 4
			If Self.Node.Pos.Y * TWorld.TILEHEIGHT < Self.SPos.Y Then Self.SPos.Y:- 4
			If Self.Node.Pos.Y * TWorld.TILEHEIGHT > Self.SPos.Y Then Self.SPos.Y:+ 4
			If Self.Node.Pos.X * TWorld.TILEWIDTH = Self.SPos.X And Self.Node.Pos.Y * TWorld.TILEHEIGHT = Self.SPos.Y Then Self.Node = Null
		EndIf
	End Method
	
	Method Render(scrollx:Int, scrolly:Int)
		Local x:Int = scrollx + Self.SPos.X, y:Int = scrolly + Self.SPos.Y
		DrawImage Sprite, x, y, Self.Char
		SetColor 0, 0, 0
		x:+ TWorld.TILEWIDTH * 0.5 - TextWidth(Self.Health) * 0.5
		DrawTextOutline Self.Health, x, y - 8
		SetColor 80, 100, 200
		If Self.Team = 1 Then SetColor 200, 100, 80
		DrawText Self.Health, x, y - 8
		SetColor 255, 255, 255
	End Method
End Type

Type TNode
	Global Reflection:TImage
	Global Selection:TImage
	Field Pos:TPoint = New TPoint
	Field Parent:TNode
	Field Anim:Int
	
	Function Create:TNode(parent:TNode, x:Int, y:Int)
		Local created:TNode = New TNode
		created.Parent = parent
		created.Pos = TPoint.Create(x, y)
		Return created
	End Function
	
	Function Load()
		Self.Reflection = LoadImage("graphics/reflection.png", 0)
		Self.Selection = LoadImage("graphics/selection.png", 0)
	End Function
	
	Method Render(reflect:Byte, scrollx:Int, scrolly:Int)
		Local x:Int = Self.Pos.X * TWorld.TILEWIDTH
		Local y:Int = Self.Pos.Y * TWorld.TILEHEIGHT
		
		SetAlpha 0.5
		DrawImage Selection, scrollx + x, scrolly + y
		If reflect Then
			If Self.Parent <> Null Then Self.Parent.Render(reflect, scrollx, scrolly)
			SetAlpha 0.2
			SetColor 255, 255, 255
			Self.Anim:+ 1
			If Self.Anim >= 0 Then Self.Anim:- 64
			SetViewport scrollx + x + 2, scrolly + y + 2, TWorld.TILEWIDTH - 4, TWorld.TILEHEIGHT - 4
			TileImage Self.Reflection, scrollx + x + anim, scrolly + y + anim
			SetViewport 0, 0, GraphicsWidth(), GraphicsHeight()
		EndIf
		SetAlpha 1
	End Method
End Type

Type TParticle
	Field Pos:TPoint, Vel:TPoint
	Field R:Int, G:Int, B:Int
	Field Life:Float
	
	Function Create(x:Int, y:Int, vx:Int, vy:Int, r:Int = 0, g:Int = 0, b:Int = 0)
		Local created:TParticle = New TParticle
		created.Pos = TPoint.Create(x, y)
		created.Vel = TPoint.Create(vx, vy)
		created.R = r
		created.G = g
		created.B = b
		TWorld.Particles.AddLast(created)
	End Function
	
	Method Update()
		Self.Pos.X:+ Self.Vel.X
		Self.Pos.Y:+ Self.Vel.Y
		Self.Life:+ 1
		If Self.Life >= 40 Then TWorld.Particles.Remove(Self)
	End Method
	
	Method Render(scrollx:Int, scrolly:Int)
		SetAlpha 1 - Life / 40.0
		SetColor Self.R, Self.G, Self.B
		DrawRect scrollx + Self.Pos.X, scrolly + Self.Pos.Y, 2 + Life / 4.0, 2 + Self.Life / 4.0
	End Method
End Type

Type TPoint
	Field X:Int, Y:Int
	
	Function Create:TPoint(x:Int, y:Int)
		Local created:TPoint = New TPoint
		created.X = x
		created.Y = y
		Return created
	End Function
End Type

Function DrawTextOutline(text:String, x:Int, y:Int)
	SetColor 0, 0, 0
	For Local alpha:Int = 0 Until 360 Step 45
		DrawText text, x + Sin(alpha), y - Cos(alpha)
	Next
End Function

AppTitle:String = "Horstcraft I"
SetGraphicsDriver GLMax2DDriver()
Graphics 640, 400, 0, 60
SetBlend ALPHABLEND
SetMaskColor 127, 127, 127
SeedRnd MilliSecs()

Local map:String = "default.map"
If AppArgs.Dimensions()[0] > 1 Then map = AppArgs[1]
TWorld.Load("maps/" + map)
Local world:TWorld = New TWorld
world.UpdateWaterMap()

Repeat
	Cls
	world.Update()
	world.Render()
	Flip
Until AppTerminate()
